#[repr(C)]pub struct tOpenArchiveData {
pub ArcName: *mut c_char,
pub OpenMode: c_int,
pub OpenResult: c_int,
pub CmtBuf: *mut c_char,
pub CmtBufSize: c_int,
pub CmtSize: c_int,
pub CmtState: c_int,
}Expand description
tOpenArchiveData is used in OpenArchive.
typedef struct {
char* ArcName;
int OpenMode;
int OpenResult;
char* CmtBuf;
int CmtBufSize;
int CmtSize;
int CmtState;
} tOpenArchiveData;Description
ArcName contains the name of the archive to open.
OpenMode is set to one of the following values:
| Constant | Value | Description |
|---|---|---|
| PK_OM_LIST | 0 | Open file for reading of file names only |
| PK_OM_EXTRACT | 1 | Open file for processing (extract or test) |
OpenResult used to return one of the error values if an error occurs.
The Cmt* variables are for the file comment. They are currently not used by Total Commander, so may be set to NULL.
Notes:
If the file is opened with OpenMode==PK_OM_LIST, ProcessFile will never be called by Total Commander.
The Unicode version of this function uses WCHAR* instead of char* for the text fields.
Fields
ArcName: *mut c_charOpenMode: c_intOpenResult: c_intCmtBuf: *mut c_charCmtBufSize: c_intCmtSize: c_intCmtState: c_intAuto Trait Implementations
impl Freeze for tOpenArchiveData
impl RefUnwindSafe for tOpenArchiveData
impl !Send for tOpenArchiveData
impl !Sync for tOpenArchiveData
impl Unpin for tOpenArchiveData
impl UnsafeUnpin for tOpenArchiveData
impl UnwindSafe for tOpenArchiveData